package io.emqx.exhook.MongoDB.reflect.impl;

import io.emqx.exhook.MongoDB.reflect.*;
import static io.emqx.exhook.MongoDB.reflect.ReflectUtils.*;
import java.lang.reflect.Field;
import java.lang.reflect.Method;


/**
 * @Description Field生成带setter、getter方法的BeanField
 * @Author Sugar
 * @Date 2020/10/9 11:53
 */
public class BeanFieldConverter implements IFieldConverter<BeanField> {
    @Override
    public BeanField convert(Class clazz, Field field) {
        Method setter = null;
        String setterName = getSetterMethodName(field);
        try {
            setter = clazz.getMethod(setterName, field.getType());
        } catch (NoSuchMethodException e) {
            //如果没有参数为该属性的数据类型的set方法，则再找名字相同的其他set方法，以兼容重写了set方法的情况
            setter = getSetterMethod(clazz, field);
            if (setter == null) {
                System.out.println("!No setter method: {"+field.getName()+"}");
            }
        }
        Method getter = null;
        String getterName = getGetterMethodName(field);
        try {
            getter = clazz.getMethod(getterName);
        } catch (NoSuchMethodException e) {
            //如果boolean类型属性无对应的isXxx()方法，则再找对应的getXxx()方法
            if (getterName.startsWith("is")) {
                getterName = "get" + getterName.substring(2);
                try {
                    getter = clazz.getMethod(getterName);
                } catch (NoSuchMethodException ex) {
                    System.out.println("!No getter method: {"+field.getName()+"}");
                    ex.printStackTrace();
                }
            } else {
                System.out.println("!No getter method: {"+field.getName()+"}");
            }
        }
        return new BeanField(field, setter, getter);
    }
}